package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

public class Wokno extends JFrame {
    
    public static void main(String[] args) {        
        File fold = null;
        if (args.length==0) {
            fold=new File(".");
            new Wokno(fold);
        } else {
            String zadano=args[0];
            if ((zadano.toUpperCase().matches(".+\\.JPG")) || (zadano.toUpperCase().matches(".+\\.JPEG"))) {
                Wokno.zobraz(new File(zadano),false);
                return;
            } else {
                fold=new File(zadano);
                new Wokno(fold);
            }
        }              
    }
    
    private JPanel folderChooser,imagesPreview;;
    private JSplitPane divider;
    private JTree tree; 
    private JScrollPane roloFolders,roloImages;
    private JPanel previews;
    
    
    public File currentFolder;
    
    
    public Wokno(File fold) {
        super("ImageViewer");
        this.currentFolder=fold;
        
        init();
        makeLayout();       
        addListeners();
        
        setVisible(true);
    }
    
    private void addListeners() {
        this.addKeyListener(new KeyAdapter() {
           public void keyTyped(KeyEvent ev) {
               if (ev.getKeyCode()==ev.VK_ESCAPE) System.exit(0);
           } 
        });
    }
    
    public static void zobraz(File f,boolean shodit) {
        Viewer pom=new Viewer(f);
        pom.setDefaultCloseOperation(shodit ? DISPOSE_ON_CLOSE:EXIT_ON_CLOSE);
        pom.setVisible(true);
    }
    
    private void makeLayout() {
        initComponents();
                
        folderChooser.setLayout(new GridLayout(1,1));
        folderChooser.add(roloFolders);
        
        imagesPreview.setLayout(new GridLayout(1,1));
        imagesPreview.add(roloImages);
        
        setLayout(new GridBagLayout());
        GridBagConstraints cons=new GridBagConstraints();
        cons.weightx=1;
        cons.weighty=1;
        cons.fill=cons.BOTH;
        add(divider,cons);
        
        validate();
    }
    
    // vezme dany soubor a vytvori z nej preview obrazku
    public static JPanel createPrev(File f,int celkem,int poradi) {
        JPanel pan=new JPanel();
        pan.setLayout(new GridBagLayout());
        GridBagConstraints cons=new GridBagConstraints();
        pan.setBackground(Color.white);
        
        try {
            BufferedImage img=ImageIO.read(f);
            JLabel labImg=new JLabel(new ImageIcon(img));
            JLabel labTxt=new JLabel(f.getName()+" "+f.length()/1024+"kB "+img.getWidth()+"x"+img.getHeight()+" "+poradi+"/"+(celkem==-1 ? "1":celkem));
            
           /* Date date=new Date();
            date.setTime(f.lastModified());
            labTxt.setText(labTxt.getText()+" "+date.getDay()+". "+date.getMonth()+". "+date.getYear()+" "+date.getHours()+":"+date.getMinutes());
            */
            
            cons.gridwidth=cons.REMAINDER;
            cons.gridy=1;
            pan.add(labImg,cons);
            cons.gridwidth=1;
            cons.gridy=2;
            pan.add(labTxt,cons);
            
            if (celkem!=-1) {
                final File pomf=f;
                pan.addMouseListener(new MouseAdapter() {
                    public void mouseClicked(MouseEvent ev) {
                        if (ev.getClickCount()==2) Wokno.zobraz(pomf,true);
                    } 
                });
            }
            
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(new JDialog(),ex.getMessage());
        } 
        
        return pan;
    }
    
    // projede adresar a zobrazi obrazky do panelu previews
    private void scan(File folder) {
        previews.removeAll();
                
        File[] files=folder.listFiles();        
        ArrayList<File> tmp=new ArrayList<File>();
        
        if (files==null) return;
        for (int i=0;i<files.length;i++)
            if ((files[i].getName().toUpperCase().endsWith(".JPG")) || (files[i].getName().toUpperCase().endsWith(".JPEG"))) {
                tmp.add(files[i]);
             }
        
         System.out.println("scanned");   
         previews.setLayout(new GridBagLayout());
         GridBagConstraints cons=new GridBagConstraints();
         
         JPanel[] vrat=new JPanel[tmp.size()];
         for (int i=0;i<vrat.length;i++) {
             cons.gridy=i;
             previews.add(Wokno.createPrev(tmp.get(i),vrat.length,i+1),cons);
         }
             
        
         previews.validate();
         imagesPreview.validate();
         
    }
    
    private void setNewPreviews(TreePath path) {
        String cesta=((File)path.getLastPathComponent()).getAbsolutePath();
        scan((File)path.getLastPathComponent());
     
    }
    
    private void initComponents() {
        folderChooser=new JPanel();
        imagesPreview=new JPanel();
        divider=new JSplitPane();
        tree=new JTree();
        roloFolders=new JScrollPane();
        roloImages=new JScrollPane();
        previews=new JPanel();
       
        roloFolders.setViewportView(tree);
        roloImages.setViewportView(previews);
        
        tree.setModel(new MyTreeModel(this));
        tree.setShowsRootHandles(true);
        tree.setExpandsSelectedPaths(true);
        
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                setNewPreviews(tree.getSelectionPath());
            }
            
        });
        
        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setCellRenderer(new MyTreeCellRenderer());
        tree.validate();
        
        divider.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
        divider.setDividerSize(5);
        
        divider.setLeftComponent(folderChooser);
        divider.setRightComponent(imagesPreview);
        divider.setDividerLocation(250);
        divider.setOneTouchExpandable(true);
        
        scan(currentFolder);  
    }
    
    private void init() {
        setSize(800,600);
        setLocationRelativeTo(null);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        getContentPane().setBackground(Color.white);
    }
}
